-- SpeedWarp
declare('SpeedWarp', SpeedWarp or {})
SpeedWarp.version = 1.0
SpeedWarp.timer = Timer()
SpeedWarp.active = true
SpeedWarp.short_delay = 2
SpeedWarp.long_delay = 100
SpeedWarp.delay = SpeedWarp.long_delay
SpeedWarp.factor = 3
SpeedWarp.max = 179
SpeedWarp.orig = Game.GetCVar('fov')
SpeedWarp.fov_is_fine = false

function SpeedWarp:run()
	if self.active then
		self:action()
		self.timer:SetTimeout(self.delay, function() self:run() end)
	end
end

function SpeedWarp.cubic_interp(s, e, x)
	return( 2*(s-e)*x^3 - 3*(s-e)*x^2 + s)
end

function SpeedWarp:action()
	local speed = GetActiveShipSpeed()
	local max_speed = GetActiveShipMaxSpeed()
	local fov = self.orig
	
	--only run fast when necessary
	if speed > max_speed - 5 then
		self.delay = self.short_delay
	else
		self.delay = self.long_delay
	end
	
	if speed > max_speed then
		local x = (speed - max_speed)/(self.factor*max_speed)
		--clamp fov to range
		x = math.min(x, 1)
		x = math.max(x, 0)
		fov = self.cubic_interp(self.orig, self.max, x)
		gkinterface.SetFOV(fov)
		self.fov_is_fine = false
	else
		--don't want to constantly set the fov if we don't have to, because then we can't use the zoom feature
		if not self.fov_is_fine then
			gkinterface.SetFOV(self.orig)
			self.fov_is_fine = true
		end
	end
end

function SpeedWarp:enable(silent)
	self.active = true
	self:run()
	if not silent then
		print("\127FFFFFFSpeedWarp enabled\127o")
	end
end

function SpeedWarp:disable(silent)
	self.active = false
	gkinterface.SetFOV(self.orig)
	if not silent then
		print("\127FFFFFFSpeedWarp disabled\127o")
	end
end

function SpeedWarp.proc(_,data)
	if (data == nil) then
		if SpeedWarp.active then
			SpeedWarp:disable()
		else
			SpeedWarp:enable()
		end
	elseif (data[1] == "delay" and data[2] ~= nil) then
		SpeedWarp.short_delay = tonumber(data[2])
	elseif (data[1] == "max" and data[2] ~= nil) then
		SpeedWarp.max = tonumber(data[2])
	elseif (data[1] == "factor" and data[2] ~= nil) then
		SpeedWarp.factor = tonumber(data[2])
	elseif (data[1] == "settings") then
		print("\127FFFFAASpeedWarp \127o\127FFFFFFcurrent settings:\127o")
		if SpeedWarp.active then
			print("\127888888* active: \127o\127FFFFFFtrue\127o")
		else
			print("\127888888* active: \127o\127FFFFFFfalse\127o")
		end
		print("\127888888* delay: \127o\127FFFFFF"..SpeedWarp.short_delay.."\127o")
		print("\127888888* max: \127o\127FFFFFF"..SpeedWarp.max.."\127o")
		print("\127888888* factor: \127o\127FFFFFF"..SpeedWarp.factor.."\127o")
	else
		print("\127FF6600SpeedWarp \127o\127FFFFFFversion "..SpeedWarp.version.."\127o")
		print("\127FFFFAAcommands:\127o")
		print("\127FFFFFF/speedwarp    \127o\127888888 enable/disable\127o")
		print("\127FFFFFF/speedwarp settings    \127o\127888888 show current settings\127o")
		print("\127FFFFFF/speedwarp delay <n>    \127o\127888888 set delay to n milliseconds\127o")
		print("\127FFFFFF/speedwarp max <n>    \127o\127888888 set max fov to n degrees\127o")
		print("\127FFFFFF/speedwarp factor <n>    \127o\127888888 smaller n results in a faster fov increase\127o")
	end
end

RegisterUserCommand("speedwarp",SpeedWarp.proc)
SpeedWarp:enable(true)